function [Fitted,Residual,Median_est_fit,Median_est_residual]=varfitandresidual(X,D,K,Pires,IncludeConstant,ThinOutFreq)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PURPOSE: 	Estimating the in-sample fit of the Bayesian VAR model
%
% MODEL:    x_{t}=PI_{1}*x_{t-1}+...+PI_{K}*x_{t-K}+PHI*d_{t}+eps_{t}
%									
% INPUT:	X: 		    (T-by-p) matrix with T observations on p modelled variables.
%			D: 		    (T-by-q) matrix with T observations on q
%			             non-modelled variables. The program adds a column of ones for the constant in the process
%			K:          (scalar) number of lags in the VAR.
%           Pires:      (niter-by-(p^2)*k) matrix of draws from the posterior of
%                       VAR coefficients.
%           actual_fitted    (scalar) if actual_fitted==1, then the time series in X
%                       and their fitted values are plotted.
%           crossplot   (scalar) if crossplot==1, the scatter plot of
%                       fitted versus actual are plotted
%
% OUTPUT:   Fitted      (T-p-niter) array with fitted values of the
%                       timeseries. Fitted{i} contains the fit based on the ith
%                       posterior draw.
%           Residual    (T-p-niter) array with residuals values of the
%                       timeseries. Fitted{i} contains the residuals based on the ith
%                       posterior draw.
%           median_est_fit  (T-by-p) matrix of estimated fitted values
%                       based on the posterior median of the model parameters.
%           Median_est_residual (T-by-p) matrix of estimated residuals
%                       based on the posterior median of the model parameters.
%
% AUTHOR:	Mattias Villani, Research Department, Sveriges Riksbank and Department of Statistics, Stockholm University.
%		    E-mail: mattias.villani@riksbank.se.
%
% DATE:		2003-10-30
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Transform data to multivariate regression form: Y=XB+E.
[T,p]=size(X);
if IncludeConstant>0
    D=[D ones(T,1)];
end
[Y,X,D]=transform_to_multireg(X,D,K);
[T,p]=size(Y);
q=size(D,2);
niter=size(Pires,1);

% Thining out the posterior sample
IterLoop=ceil(1:ThinOutFreq:niter);
niter=length(IterLoop);

Fitted=zeros(T,p,niter);
Residual=zeros(T,p,niter);

% Computes posterior distribution of fitted values
Count=0;
for i=IterLoop
    Count=Count+1;
    Pi=invvec(Pires(Count,:)',p*K+q);
    Fitted(:,:,Count)=[X D]*Pi;
    Residual(:,:,Count)=Y-Fitted(:,:,i);
end

% Computes estimated fitted values using the posterior median
Median_est_fit=zeros(T,p);
if size(Pires,1)==1 % Only one posterior draw of ML estimates
    medianPi=invvec(Pires',p*K+q);
else
    medianPi=invvec(median(Pires)',p*K+q);
end
for h=1:T
    Median_est_fit(h,:)=[X(h,:) D(h,:)]*medianPi;
end

Median_est_residual=Y-[X D]*medianPi;